// ***************************************
// SourceBoost IDE plugins header file
//
// Author(s): Pavel Baranov, David Hobday
//
// Copyright (C) 2002-2010 Pavel Baranov
// Copyright (C) 2002-2010 David Hobday
//
// This code may be freely used to develop
// Plugins for the SourceBoost IDE
// ***************************************

// Different defines before including this file cause different
// code to be generated

#ifndef _PLUGINAPI_H_
#define _PLUGINAPI_H_

typedef void* HANDLE;
typedef void (*CLKEXECCALLBACK)( void* pParam, long clk );
typedef void (*PINCHGCALLBACK)( HANDLE hPin, int Vmv, void* pParam, int iParam );
typedef void (*REGCHGCALLBACK)( HANDLE hReg, void* pParam, int iParam );

#define PLUGIN_API_VERSION 300

//////////////////////////////////////////
// Macros useful for callback functions - 
// For use with classes and there member functions
//////////////////////////////////////////
#define DECLARE_PINVCHANGECALLBACKFUNC( cbfclass , cbf ) \
	static void Static##cbf ( HANDLE hPin, int Vmv, void* pthis, int iParam )\
	{ \
		( (##cbfclass *)pthis)-> ##cbf ( hPin, Vmv, iParam ); \
	}

#define PINVCHANGEDCALLBACKFUNC( cbfclass , cbf ) cbfclass::Static##cbf


// define _PLUGIN_IMPORT_ for IDE include
#ifdef _PLUGIN_IMPORT_
	#define PLUGIN_API __declspec(dllimport)
#else
	#define PLUGIN_API __declspec(dllexport)
#endif

///////////////////////////////////////////////////////
// Function prototypes for Functions that the IDE will call
///////////////////////////////////////////////////////

PLUGIN_API int plgVersion( void );
PLUGIN_API int plgNumber( void );
PLUGIN_API HWND plgCreate( HWND parent, void* (*SupplyFuncPtr)(const char*), int num );
PLUGIN_API BOOL plgStart( HWND parent );
PLUGIN_API void plgStop( HWND parent );
PLUGIN_API BOOL plgFree( HWND parent );
PLUGIN_API LPCTSTR plgName( HWND parent );
PLUGIN_API void plgClockRate( HWND parent, long clockRate_Hz );
PLUGIN_API void plgConfigure( HWND parent );
PLUGIN_API int plgSave( HWND parent, BYTE *buf );
PLUGIN_API BOOL plgLoad( HWND parent, BYTE *buf, int len );
PLUGIN_API int plgSaveState( HWND parent, BYTE *buf );
PLUGIN_API BOOL plgLoadState( HWND parent, BYTE *buf, int len );
PLUGIN_API void plgReset( HWND parent );
PLUGIN_API void plgStep( HWND parent, short sDirection );
PLUGIN_API void plgClock( HWND parent, long clk, void*pParam );
PLUGIN_API void plgInstr( HWND parent, long clk, void*pParam );



#endif // _PLUGINAPI_H_






#ifdef _PLUGIN_FUNC_PROTOTYPE_
	#undef _PLUGIN_FUNC_PROTOTYPE_
	

	//////////////////////////////////////////////////////
	// Macros that create conditional statement to provide 
	// function pointer - that which will be supplied by
	// the IDE.
	//////////////////////////////////////////////////////
	#undef DECLARE_PLUGINFUNC // incase already defined
	#define DECLARE_PLUGINFUNC( interfacefuncName, argList ) void interfacefuncName argList;

	#undef PLUGINFUNC_START
	#define PLUGINFUNC_START

	#undef PLUGINFUNC_END
	#define PLUGINFUNC_END

#endif // _PLUGIN_FUNC_PROTOTYPE_



#ifdef _PLUGIN_FUNC_PTRS_SUPPLY_
	#undef _PLUGIN_FUNC_PTRS_SUPPLY_
	//////////////////////////////////////////////////////
	// Macros that create conditional statement to provide 
	// function pointer - that which will be supplied by
	// the IDE.
	//////////////////////////////////////////////////////
	#undef DECLARE_PLUGINFUNC // incase already defined
	#define DECLARE_PLUGINFUNC( interfacefuncName, argList )\
	{\
		void ( *ptr ) argList = interfacefuncName; /* type check before its cast away */ \
		if ( stricmp( funcName, #interfacefuncName ) == 0 )\
			return ptr; \
	}

	#undef PLUGINFUNC_START
	#define PLUGINFUNC_START \
		void* SupplyFuncPtr( const char* funcName ) \
		{ \

	#undef PLUGINFUNC_END
	#define PLUGINFUNC_END \
			return NULL; \
		}

	
#endif // _PLUGIN_FUNC_PTRS_SUPPLY_

#ifdef _PLUGIN_FUNC_PTRS_DECLARE_
	#undef _PLUGIN_FUNC_PTRS_DECLARE_
	
	//////////////////////////////////////////////////////
	// Macros to declare functions (pointers) that plugin can call 
	//////////////////////////////////////////////////////
	// This DECLARE_PLUGINFUNC( GetRegHandle, ( char* name, HANDLE* hReg ) ) macro generates the following code
	// typedef ( *typeGetRegHandle ) ( char* name, HANDLE* hReg );
	// typeGetRegHandle GetRegHandle;
	#undef DECLARE_PLUGINFUNC // incase already defined
	#define DECLARE_PLUGINFUNC( funcName, argList ) \
		void (*funcName) argList; 

	#undef PLUGINFUNC_START
	#define PLUGINFUNC_START

	#undef PLUGINFUNC_END
	#define PLUGINFUNC_END

#endif // _PLUGIN_FUNC_PTRS_DECLARE_

#ifdef _PLUGIN_FUNC_PTRS_RESOLVE_
	#undef _PLUGIN_FUNC_PTRS_RESOLVE_

	//////////////////////////////////////////////////////
	// Macros to create funcion to resolve function pointers
	//////////////////////////////////////////////////////

	// This DECLARE_PLUGINFUNC( GetRegHandle ); macro generates the following code
	// GetRegHandle = (typeGetRegHandle) SupplyFuncPtr( "GetRegHandle" );
	#undef DECLARE_PLUGINFUNC
	#define DECLARE_PLUGINFUNC( funcName, argList )\
		funcName = (void (*) argList) SupplyFuncPtr( #funcName );\
		if ( funcName == NULL ) badFunction = true;

	#undef PLUGINFUNC_START
	#define PLUGINFUNC_START \
		bool ResolveFuncPtrs( void*(SupplyFuncPtr)( const char* ) )\
		{ \
			bool badFunction = false;

	#undef PLUGINFUNC_END
	#define PLUGINFUNC_END \
			return badFunction ? false: true;\
		}

#endif // _PLUGIN_FUNC_PTRS_RESOLVE_



// Declare pointers as external
#ifndef DECLARE_PLUGINFUNC

	//////////////////////////////////////////////////////
	// Macros to declare extern functions (pointers) that plugin can call 
	//////////////////////////////////////////////////////
	// This DECLARE_PLUGINFUNC( GetRegHandle, ( char* name, HANDLE* hReg ) ) macro generates the following code
	// typedef ( *typeGetRegHandle ) ( char* name, HANDLE* hReg );
	// typeGetRegHandle GetRegHandle;
	#undef DECLARE_PLUGINFUNC // incase already defined
	#define DECLARE_PLUGINFUNC( funcName, argList ) \
		extern void (*funcName) argList; 

	#undef PLUGINFUNC_START
	#define PLUGINFUNC_START

	#undef PLUGINFUNC_END
	#define PLUGINFUNC_END

#endif


//////////////////////////////////////////////////////
// Declare functions (pointers) that plugin can call 
//////////////////////////////////////////////////////
PLUGINFUNC_START

DECLARE_PLUGINFUNC( GetRegHandle, ( char* name, HANDLE* hReg ) )
DECLARE_PLUGINFUNC( GetRegVal, ( HANDLE  hReg, int* val ) )
DECLARE_PLUGINFUNC( SetRegVal, ( HANDLE  hReg, int val ) )
DECLARE_PLUGINFUNC( AddRegChangeCallBack,( HANDLE hReg, REGCHGCALLBACK pfunc, void* pParam, int iParam ) )
DECLARE_PLUGINFUNC( RemoveRegChangeCallBack, ( HANDLE hReg, REGCHGCALLBACK pfunc, void* pParam, int iParam ) )

DECLARE_PLUGINFUNC( MakeConnectionToPin,( const char* pinName, HANDLE* hConnection ) )
DECLARE_PLUGINFUNC( ReleaseConnection,( HANDLE hConnection ) )
DECLARE_PLUGINFUNC( SetConnectionV, ( HANDLE  hConnection, int mv ) )
DECLARE_PLUGINFUNC( SetConnectionR, ( HANDLE  hConnection, int ohm ) )
DECLARE_PLUGINFUNC( GetPinV, ( HANDLE  hConnection, int* mv ) )
DECLARE_PLUGINFUNC( AddPinVChangeCallBack,( HANDLE hConnection, PINCHGCALLBACK pfunc, void* pParam, int iParam ) )
DECLARE_PLUGINFUNC( RemovePinVChangeCallBack, ( HANDLE hConnection, PINCHGCALLBACK pfunc, void* pParam, int iParam ) )

DECLARE_PLUGINFUNC( GetClk, ( long* clk ) )
DECLARE_PLUGINFUNC( Lock, ( void ) )
DECLARE_PLUGINFUNC( Unlock, ( void ) )

PLUGINFUNC_END







